      SUBROUTINE PDLAQGE( M, N, A, IA, JA, DESCA, R, C, ROWCND, COLCND,
     $                    AMAX, EQUED )
*
*  -- ScaLAPACK auxilliary routine (version 1.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     February 28, 1995
*
*     .. Scalar Arguments ..
      CHARACTER          EQUED
      INTEGER            IA, JA, M, N
      DOUBLE PRECISION   AMAX, COLCND, ROWCND
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( * )
      DOUBLE PRECISION   A( * ), C( * ), R( * )
*     ..
*
*  Purpose
*  =======
*
*  PDLAQGE equilibrates a general M-by-N distributed matrix
*  sub( A ) = A(IA:IA+M-1,JA:JA+N-1) using the row and scaling
*  factors in the vectors R and C.
*
*  Notes
*  =====
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN  EXPLANATION
*  --------------- ---------- ------------------------------------------
*  M_A    (global) DESCA( 1 ) The number of rows in the distributed
*                             matrix.
*  N_A    (global) DESCA( 2 ) The number of columns in the distributed
*                             matrix.
*  MB_A   (global) DESCA( 3 ) The blocking factor used to distribute
*                             the rows of the matrix.
*  NB_A   (global) DESCA( 4 ) The blocking factor used to distribute
*                             the columns of the matrix.
*  RSRC_A (global) DESCA( 5 ) The process row over which the first row
*                             of the matrix is distributed.
*  CSRC_A (global) DESCA( 6 ) The process column over which the first
*                             column of the matrix is distributed.
*  CTXT_A (global) DESCA( 7 ) The BLACS context handle, indicating the
*                             BLACS process grid A is distributed over.
*                             The context itself is global, but the handle
*                             (the integer value) may vary.
*  LLD_A  (local)  DESCA( 8 ) The leading dimension of the local array
*                             storing the local blocks of the distri-
*                             buted matrix A. LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*  Arguments
*  =========
*
*  M       (global input) INTEGER
*          The number of rows to be operated on i.e the number of rows
*          of the distributed submatrix sub( A ). M >= 0.
*
*  N       (global input) INTEGER
*          The number of columns to be operated on i.e the number of
*          columns of the distributed submatrix sub( A ). N >= 0.
*
*  A       (local input/local output) DOUBLE PRECISION pointer into the
*          local memory to an array of dimension (LLD_A,LOCq(JA+N-1))
*          containing on entry the M-by-N matrix sub( A ). On exit,
*          the equilibrated distributed matrix.  See EQUED for the
*          form of the equilibrated distributed submatrix.
*
*  IA      (global input) INTEGER
*          A's global row index, which points to the beginning of the
*          submatrix which is to be operated on.
*
*  JA      (global input) INTEGER
*          A's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCA   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix A.
*
*  R       (local input) DOUBLE PRECISION array, dimension LOCp(M_A)
*          The row scale factors for sub( A ). R is aligned with the
*          distributed matrix A, and replicated in every process
*          column. R is tied to the distributed matrix A.
*
*  C       (local input) DOUBLE PRECISION array, dimension LOCq(N_A)
*          The column scale factors of sub( A ). C is aligned with the
*          distributed matrix A, and replicated in every process
*          column. C is tied to the distributed matrix A.
*
*  ROWCND  (global input) DOUBLE PRECISION
*          The global ratio of the smallest R(i) to the largest R(i),
*          IA <= i <= IA+M-1.
*
*  COLCND  (global input) DOUBLE PRECISION
*          The global ratio of the smallest C(i) to the largest C(i),
*          JA <= j <= JA+N-1.
*
*  AMAX    (global input) DOUBLE PRECISION
*          Absolute value of largest distributed submatrix entry.
*
*  EQUED   (global output) CHARACTER
*          Specifies the form of equilibration that was done.
*          = 'N':  No equilibration
*          = 'R':  Row equilibration, i.e., sub( A ) has been pre-
*                  multiplied by diag(R(IA:IA+M-1)),
*          = 'C':  Column equilibration, i.e., sub( A ) has been post-
*                  multiplied by diag(C(JA:JA+N-1)),
*          = 'B':  Both row and column equilibration, i.e., sub( A )
*                  has been replaced by
*                  diag(R(IA:IA+M-1)) * sub( A ) * diag(C(JA:JA+N-1)).
*
*  Internal Parameters
*  ===================
*
*  THRESH is a threshold value used to decide if row or column scaling
*  should be done based on the ratio of the row or column scaling
*  factors.  If ROWCND < THRESH, row scaling is done, and if
*  COLCND < THRESH, column scaling is done.
*
*  LARGE and SMALL are threshold values used to decide if row scaling
*  should be done based on the absolute size of the largest matrix
*  element.  If AMAX > LARGE or AMAX < SMALL, row scaling is done.
*
*  =====================================================================
*
*     .. Parameters ..
      DOUBLE PRECISION   ONE, THRESH
      PARAMETER          ( ONE = 1.0D+0, THRESH = 0.1D+0 )
*     ..
*     .. Local Scalars ..
      INTEGER            I, IACOL, IAROW, ICOFF, ICTXT, IIA, IOFFA,
     $                   IROFF, J, JJA, LDA, MP, MYCOL, MYROW, NPCOL,
     $                   NPROW, NQ
      DOUBLE PRECISION   CJ, LARGE, SMALL
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, INFOG2L
*     ..
*     .. External Functions ..
      INTEGER            NUMROC
      DOUBLE PRECISION   PDLAMCH
      EXTERNAL           NUMROC, PDLAMCH
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MOD
*     ..
*     .. Executable Statements ..
*
*     Quick return if possible
*
      IF( M.LE.0 .OR. N.LE.0 ) THEN
         EQUED = 'N'
         RETURN
      END IF
*
*     Get grid parameters and compute local indexes
*
      ICTXT = DESCA( 7 )
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
      CALL INFOG2L( IA, JA, DESCA, NPROW, NPCOL, MYROW, MYCOL, IIA, JJA,
     $              IAROW, IACOL )
      IROFF = MOD( IA-1, DESCA( 3 ) )
      ICOFF = MOD( JA-1, DESCA( 4 ) )
      MP = NUMROC( M+IROFF, DESCA( 3 ), MYROW, IAROW, NPROW )
      NQ = NUMROC( N+ICOFF, DESCA( 4 ), MYCOL, IACOL, NPCOL )
      IF( MYROW.EQ.IAROW )
     $   MP = MP - IROFF
      IF( MYCOL.EQ.IACOL )
     $   NQ = NQ - ICOFF
      LDA = DESCA( 8 )
*
*     Initialize LARGE and SMALL.
*
      SMALL = PDLAMCH( ICTXT, 'Safe minimum' ) /
     $        PDLAMCH( ICTXT, 'Precision' )
      LARGE = ONE / SMALL
*
      IF( ROWCND.GE.THRESH .AND. AMAX.GE.SMALL .AND. AMAX.LE.LARGE )
     $     THEN
*
*        No row scaling
*
         IF( COLCND.GE.THRESH ) THEN
*
*           No column scaling
*
            EQUED = 'N'
*
         ELSE
*
*           Column scaling
*
            IOFFA = (JJA-1)*LDA
            DO 20 J = JJA, JJA+NQ-1
               CJ = C( J )
               DO 10 I = IIA, IIA+MP-1
                  A( IOFFA + I ) = CJ*A( IOFFA + I )
   10          CONTINUE
               IOFFA = IOFFA + LDA
   20       CONTINUE
            EQUED = 'C'
         END IF
*
      ELSE IF( COLCND.GE.THRESH ) THEN
*
*        Row scaling, no column scaling
*
         IOFFA = (JJA-1)*LDA
         DO 40 J = JJA, JJA+NQ-1
            DO 30 I = IIA, IIA+MP-1
               A( IOFFA + I ) = R( I )*A( IOFFA + I )
   30       CONTINUE
            IOFFA = IOFFA + LDA
   40    CONTINUE
         EQUED = 'R'
*
      ELSE
*
*        Row and column scaling
*
         IOFFA = (JJA-1)*LDA
         DO 60 J = JJA, JJA+NQ-1
            CJ = C( J )
            DO 50 I = IIA, IIA+MP-1
               A( IOFFA + I ) = CJ*R( I )*A( IOFFA + I )
   50       CONTINUE
            IOFFA = IOFFA + LDA
   60    CONTINUE
         EQUED = 'B'
*
      END IF
*
      RETURN
*
*     End of PDLAQGE
*
      END
