      SUBROUTINE PDPOTRRV( UPLO, N, A, IA, JA, DESCA, WORK )
*
*  -- ScaLAPACK routine (version 1.0) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     February 28, 1995
*
*     .. Scalar Arguments ..
      CHARACTER          UPLO
      INTEGER            IA, JA, N
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( * )
      DOUBLE PRECISION   A( * ),  WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  PDPOTRRV recomputes sub( A ) = A(IA:IA+N-1,JA:JA+N-1) from L or U
*  computed by PDPOTRF. The routine performs the Cholesky factorization
*  in reverse.
*
*  Notes
*  =====
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN  EXPLANATION
*  --------------- ---------- ------------------------------------------
*  M_A    (global) DESCA( 1 ) The number of rows in the distributed
*                             matrix.
*  N_A    (global) DESCA( 2 ) The number of columns in the distributed
*                             matrix.
*  MB_A   (global) DESCA( 3 ) The blocking factor used to distribute
*                             the rows of the matrix.
*  NB_A   (global) DESCA( 4 ) The blocking factor used to distribute
*                             the columns of the matrix.
*  RSRC_A (global) DESCA( 5 ) The process row over which the first row
*                             of the matrix is distributed.
*  CSRC_A (global) DESCA( 6 ) The process column over which the first
*                             column of the matrix is distributed.
*  CTXT_A (global) DESCA( 7 ) The BLACS context handle, indicating the
*                             BLACS process grid A is distributed over.
*                             The context itself is global, but the handle
*                             (the integer value) may vary.
*  LLD_A  (local)  DESCA( 8 ) The leading dimension of the local array
*                             storing the local blocks of the distri-
*                             buted matrix A. LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*  Arguments
*  =========
*
*  UPLO    (global input) CHARACTER
*          Specifies whether the upper or lower triangular part of the
*          symmetric distributed matrix sub( A ) is stored:
*          stored:
*          = 'U':  Upper triangular
*          = 'L':  Lower triangular
*
*  N       (global input) INTEGER
*          The number of rows and columns to be operated on, i.e. the
*          order of the distributed submatrix sub( A ). N >= 0.
*
*  A       (local input/local output) DOUBLE PRECISION pointer into the
*          local memory to an array of dimension (LLD_A, LOCq(JA+N-1)).
*          On entry, the local pieces of the factors L or U of the
*          distributed matrix sub( A ) from the Cholesky factorization.
*          On exit, the original distributed matrix sub( A ) is restored.
*
*  IA      (global input) INTEGER
*          A's global row index, which points to the beginning of the
*          submatrix which is to be operated on.
*
*  JA      (global input) INTEGER
*          A's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCA   (global and local input) INTEGER array of dimension 8
*          The array descriptor for the distributed matrix A.
*
*  WORK    (local workspace) DOUBLE PRECISION array, dimension (LWORK)
*          LWORK >= MB_A*NB_A.
*
*  =====================================================================
*
*     .. Parameters ..
      DOUBLE PRECISION   ONE, ZERO
      PARAMETER          ( ONE = 1.0D+0, ZERO = 0.0D+0 )
*     ..
*     .. Local Scalars ..
      LOGICAL            UPPER
      CHARACTER          COLBTOP, ROWBTOP
      INTEGER            IACOL, IAROW, ICTXT, IL, J, JB, JL, JN, MYCOL,
     $                   MYROW, NPCOL, NPROW
*     .. Local Arrays ..
      INTEGER            DESCW( 8 )
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, DESCSET, PDLACPY, PDLASET,
     $                   PDSYRK, PDTRMM, PTOPGET, PTOPSET
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      INTEGER            ICEIL, INDXG2P
      EXTERNAL           ICEIL, INDXG2P, LSAME
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MIN, MOD
*     ..
*     .. Executable Statements ..
*
*     Get grid parameters
*
      ICTXT = DESCA( 7 )
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
*
      CALL PTOPGET( 'Broadcast', 'Rowwise', ROWBTOP )
      CALL PTOPGET( 'Broadcast', 'Columnwise', COLBTOP )
*
      UPPER = LSAME( UPLO, 'U' )
      JN = MIN( ICEIL( JA, DESCA( 4 ) ) * DESCA( 4 ), JA+N-1 )
      JL = MAX( ( ( JA+N-2 ) / DESCA( 4 ) ) * DESCA( 4 ) + 1, JA )
      IL = MAX( ( ( IA+N-2 ) / DESCA( 3 ) ) * DESCA( 3 ) + 1, IA )
      IAROW = INDXG2P( IL, DESCA( 3 ), MYROW, DESCA( 5 ), NPROW )
      IACOL = INDXG2P( JL, DESCA( 4 ), MYCOL, DESCA( 6 ), NPCOL )
*
*     Define array descriptor for working array WORK
*
      CALL DESCSET( DESCW, DESCA( 3 ), DESCA( 4 ), DESCA( 3 ),
     $              DESCA( 4 ), IAROW, IACOL, ICTXT, DESCA( 3 ) )
*
      IF ( UPPER ) THEN
*
*       Compute A from the Cholesky factor U : A = U'*U.
*
        CALL PTOPSET( 'Broadcast', 'Rowwise', ' ' )
        CALL PTOPSET( 'Broadcast', 'Columnwise', 'S-ring' )
*
        DO 10 J = JL, JN+1, -DESCA( 4 )
*
           JB = MIN( JA+N-J, DESCA( 4 ) )
*
*          Update the trailing matrix, A = A + U'*U
*
           CALL PDSYRK( 'Upper', 'Transpose', JA+N-J-JB, JB, ONE, A, IL,
     $                  J+JB, DESCA, ONE, A, IL+JB, J+JB, DESCA )
*
*          Copy current diagonal block of A into workspace
*
           CALL PDLACPY( 'All', JB, JB, A, IL, J, DESCA, WORK, 1, 1,
     $                   DESCW )
*
*          Zero strict lower triangular part of diagonal block, to make
*          it U1.
*
           CALL PDLASET( 'Lower', JB-1, JB, ZERO, ZERO, A, IL+1, J,
     $                   DESCA )
*
*          Update the row panel U with the triangular matrix
*
           CALL PDTRMM( 'Left', 'Upper', 'Transpose', 'Non-Unit', JB,
     $                  N-J+JA, ONE, WORK, 1, 1, DESCW, A, IL, J,
     $                  DESCA )
*
*          Restore the strict lower triangular part of diagonal block.
*
           CALL PDLACPY( 'Lower', JB-1, JB, WORK, 2, 1, DESCW, A,
     $                    IL+1, J, DESCA )
*
           IL = IL - DESCA( 3 )
           DESCW( 5 ) = MOD( DESCW( 5 ) + NPROW - 1, NPROW )
           DESCW( 6 ) = MOD( DESCW( 6 ) + NPCOL - 1, NPCOL )
*
   10   CONTINUE
*
*       Handle first block separately
*
        JB = MIN( JN-JA+1, DESCA( 4 ) )
*
*       Update the trailing matrix, A = A + U'*U
*
        CALL PDSYRK( 'Upper', 'Transpose', N-JB, JB, ONE, A, IA, JA+JB,
     $               DESCA, ONE, A, IA+JB, JA+JB, DESCA )
*
*       Copy current diagonal block of A into workspace
*
        CALL PDLACPY( 'All', JB, JB, A, IA, JA, DESCA, WORK, 1, 1,
     $                DESCW )
*
*       Zero strict lower triangular part of diagonal block, to make
*       it U1.
*
        CALL PDLASET( 'Lower', JB-1, JB, ZERO, ZERO, A, IA+1, JA,
     $                DESCA )
*
*       Update the row panel U with the triangular matrix
*
        CALL PDTRMM( 'Left', 'Upper', 'Transpose', 'Non-Unit', JB,
     $               N, ONE, WORK, 1, 1, DESCW, A, IA, JA, DESCA )
*
*       Restore the strict lower triangular part of diagonal block.
*
        CALL PDLACPY( 'Lower', JB-1, JB, WORK, 2, 1, DESCW, A, IA+1,
     $                JA, DESCA )
*
      ELSE
*
*       Compute A from the Cholesky factor L : A = L*L'.
*
        CALL PTOPSET( 'Broadcast', 'Rowwise', 'S-ring' )
        CALL PTOPSET( 'Broadcast', 'Columnwise', ' ' )
*
        DO 20 J = JL, JN+1, -DESCA( 4 )
*
           JB = MIN( JA+N-J, DESCA( 4 ) )
*
*          Update the trailing matrix, A = A + L*L'
*
           CALL PDSYRK( 'Lower', 'No transpose', IA+N-IL-JB, JB, ONE, A,
     $                  IL+JB, J, DESCA, ONE, A, IL+JB, J+JB, DESCA )
*
*          Copy current diagonal block of A into workspace
*
           CALL PDLACPY( 'All', JB, JB, A, IL, J, DESCA, WORK, 1, 1,
     $                   DESCW )
*
*          Zero strict upper triangular part of diagonal block, to make
*          it L1.
*
           CALL PDLASET( 'Upper', JB, JB-1, ZERO, ZERO, A, IL, J+1,
     $                   DESCA )
*
*          Update the column panel L with the triangular matrix
*
           CALL PDTRMM( 'Right', 'Lower', 'Transpose', 'Non-Unit',
     $                   IA+N-IL, JB, ONE, WORK, 1, 1, DESCW, A, IL,
     $                   J, DESCA )
*
*          Restore the strict upper triangular part of diagonal block.
*
           CALL PDLACPY( 'Upper', JB, JB-1, WORK, 1, 2, DESCW, A,
     $                   IL, J+1, DESCA )
*
           IL = IL - DESCA( 3 )
           DESCW( 5 ) = MOD( DESCW( 5 ) + NPROW - 1, NPROW )
           DESCW( 6 ) = MOD( DESCW( 6 ) + NPCOL - 1, NPCOL )
*
   20   CONTINUE
*
*       Handle first block separately
*
        JB = MIN( JN-JA+1, DESCA( 4 ) )
*
*       Update the trailing matrix, A = A + L*L'
*
        CALL PDSYRK( 'Lower', 'No transpose', N-JB, JB, ONE, A,
     $               IA+JB, JA, DESCA, ONE, A, IA+JB, JA+JB, DESCA )
*
*       Copy current diagonal block of A into workspace
*
        CALL PDLACPY( 'All', JB, JB, A, IA, JA, DESCA, WORK, 1, 1,
     $                DESCW )
*
*       Zero strict upper triangular part of diagonal block, to make
*       it L1.
*
        CALL PDLASET( 'Upper', JB, JB-1, ZERO, ZERO, A, IA, JA+1,
     $                DESCA )
*
*       Update the column panel L with the triangular matrix
*
        CALL PDTRMM( 'Right', 'Lower', 'Transpose', 'Non-Unit', N, JB,
     $               ONE, WORK, 1, 1, DESCW, A, IA, JA, DESCA )
*
*       Restore the strict upper triangular part of diagonal block.
*
        CALL PDLACPY( 'Upper', JB, JB-1, WORK, 1, 2, DESCW, A, IA,
     $                JA+1, DESCA )
*
      END IF
*
      CALL PTOPSET( 'Broadcast', 'Rowwise', ROWBTOP )
      CALL PTOPSET( 'Broadcast', 'Columnwise', COLBTOP )
*
      RETURN
*
*     End of PDPOTRRV
*
      END
